package game_srv

import (
	"context"
	"encoding/json"
	"fmt"
	"usdk-srv-sdk/usdklib"
	"log"
	"net/http"
	"time"
)

func (g *tGameServer) init() error {
	usdk, err := usdklib.NewUSDK(g.ctx, PRODUCT_ID, PRODUCT_KEY, CALLBACK_KEY, usdklib.AreaGlobal)
	if err != nil {
		return err
	}
	g.usdk = usdk
	g.gameUser = make(map[string]*tGameUser)
	return nil
}

func (g *tGameServer) Close() {
	if g.usdk != nil {
		g.usdk = nil
	}
	if g.gameUser != nil {
		g.gameUser = nil
	}
}

func respJson(w http.ResponseWriter, v interface{}) {
	b1, _ := json.Marshal(v)
	_, _ = w.Write(b1)
	w.Header().Set("Content-type", "application/json")
	w.WriteHeader(http.StatusOK)
}

func buildResp(code int, msg string, k string, v interface{}) interface{} {
	r := map[string]interface{}{"code": code}
	if msg != "" {
		r["msg"] = msg
	}
	if k != "" {
		r[k] = v
	}
	return r
}

// 示例：关联游戏账号
func (g *tGameServer) kitGenGameUserId(channelId int, cUid string) string {
	gameUser := fmt.Sprintf("%v_%v", channelId, cUid)
	return gameUser
}

func (g *tGameServer) apiLogin(w http.ResponseWriter, r *http.Request) {
	cUid := r.FormValue("cUid")
	cName := r.FormValue("cName")
	accessToken := r.FormValue("accessToken")
	log.Printf("登录验证, cUid=%s cName=%s accessToken=%s", cUid, cName, accessToken)
	respBody, authInfo, err := g.usdk.Login(cUid, cName, accessToken)

	if respBody != nil {
		log.Println("登录结果:", string(respBody))
	}

	if err == nil && authInfo != nil && authInfo.Code == 0 {
		log.Printf("登录渠道：%v(%v)", authInfo.Channel, authInfo.ChannelId)
		gameUser := g.kitGenGameUserId(authInfo.ChannelId, authInfo.CUid)
		if u, ok := g.gameUser[gameUser]; ok {
			log.Printf("更新关联游戏账号: %s", gameUser)
			u.SdkUser.ChannelUserId = authInfo.CUid
			u.SdkUser.ChannelUserName = authInfo.CName
			u.SdkUser.Channel = authInfo.Channel
			u.SdkUser.ChannelId = authInfo.ChannelId
		} else {
			log.Printf("创建新关联游戏账号: %s", gameUser)
			g.gameUser[gameUser] = &tGameUser{
				SdkUser: tSdkUserGameUser{
					ChannelUserId:   authInfo.CUid,
					ChannelUserName: authInfo.CName,
					Channel:         authInfo.Channel,
					ChannelId:       authInfo.ChannelId,
				},
			}
		}
		respJson(w, map[string]interface{}{"code": 0, "uid": gameUser, "channel": authInfo.Channel, "channelId": authInfo.ChannelId})
		return
	} else {
		if err != nil {
			log.Println("登录失败:", err)
		} else {
			log.Println("登录失败:", authInfo)
		}
	}
	respJson(w, map[string]interface{}{"code": -1})
}

func (g *tGameServer) apiGameProp(w http.ResponseWriter, r *http.Request) {
	respJson(w, buildResp(0, "获取成功(纯演示效果，模拟获取游戏内的档位)", "props", SimpleProps))
}

func (g *tGameServer) apiOrder(w http.ResponseWriter, r *http.Request) {
	gameUid := r.FormValue("uid")
	goodsId := r.FormValue("goodsId")
	order := fmt.Sprintf("%v_%v_%d", gameUid, goodsId, time.Now().Unix()) // NOTE: 示例
	log.Printf("游戏内下单: uid=%s goodsId=%s => order=%s", gameUid, goodsId, order)
	respJson(w, buildResp(0, "", "order", order))
}

func (g *tGameServer) apiPayCallback(w http.ResponseWriter, r *http.Request) {
	data := r.FormValue("data")
	sign := r.FormValue("sign")
	ret, payData, payInfo, err := g.usdk.PayCallback(data, sign)

	if payData != nil {
		log.Printf("支付结果[%s]: %s", ret, string(payData))
	}

	if err == nil && payInfo != nil {
		log.Printf("支付回调 gameOrder: %v\t游戏的订单号", payInfo.GameOrder)
		log.Printf("支付回调 selfDefine: %v\t游戏的自定义信息", payInfo.SelfDefine)
		log.Printf("支付回调 amount: %v\t付款金额", payInfo.Amount)
		log.Printf("支付回调 payTime: %v\t支付时间", payInfo.PayTime)
		log.Printf("支付回调 goodsId: %v\t支付的道具ID", payInfo.GoodsId)
		log.Printf("支付回调 channelUid: %v\t渠道用户ID", payInfo.ChannelUid)
		log.Printf("支付回调 channel: %v\t渠道码", payInfo.Channel)
		log.Printf("支付回调 channelId: %v\t渠道ID", payInfo.ChannelId)
		log.Printf("支付回调 orderNo: %v\tUSDK的订单号", payInfo.OrderNo)

		// TODO: 这里需要进行游戏订单的合法性校验
		// 1、渠道id是否一致
		// 2、用户信息是否一致
		// 3、自定义信息是否正确
		// 4、金额档位是否正确
		// 5、......

		// 模拟加档位
		g.gameOnPay(payInfo)
		_, _ = w.Write([]byte(ret))
		w.WriteHeader(http.StatusOK)
	} else {
		if err != nil {
			log.Println("支付回调失败:", err)
		}
		_, _ = w.Write([]byte(ret))
		w.WriteHeader(http.StatusOK)
	}
}

// 示例：玩家获得道具
func (g *tGameServer) gameOnPay(p *usdklib.PayResult) {
	gameUser := g.kitGenGameUserId(p.ChannelId, p.ChannelUid)
	if u, ok := g.gameUser[gameUser]; !ok {
		log.Println("用户不存在:", gameUser)
		return
	} else {
		if u.Pack == nil {
			u.Pack = make(map[string]int)
		}
		v, ok := u.Pack[p.GoodsId]
		if !ok {
			v = 0
		}
		u.Pack[p.GoodsId] = v + 1
	}
}

func (g *tGameServer) apiGameUser(w http.ResponseWriter, r *http.Request) {
	uid := r.FormValue("uid")
	log.Printf("获取用户属性: uid=%s", uid)
	if g.gameUser != nil {
		if u, ok := g.gameUser[uid]; ok {
			log.Printf("获取用户属性: uid=%s %+v", uid, u)
			respJson(w, buildResp(0, "", "info", u))
			return
		}
	}
	log.Printf("获取用户属性: 用户不存在 uid=%s", uid)
	respJson(w, buildResp(-1, "miss user", "", nil))
}

func (g *tGameServer) apiGameUserFree(w http.ResponseWriter, r *http.Request) {
	uid := r.FormValue("uid")
	if g.gameUser != nil {
		if _, ok := g.gameUser[uid]; ok {
			delete(g.gameUser, uid)
			respJson(w, buildResp(0, "ok", "", nil))
		}
	}
	respJson(w, buildResp(-1, "miss user", "", nil))
}

func Srv() error {
	ctx := context.Background()
	game := tGameServer{ctx: ctx}
	defer game.Close()
	if err := game.init(); err != nil {
		return err
	}
	http.HandleFunc("/login", game.apiLogin)
	http.HandleFunc("/game_prop", game.apiGameProp)
	http.HandleFunc("/order", game.apiOrder)
	http.HandleFunc("/payC", game.apiPayCallback)
	http.HandleFunc("/game_user", game.apiGameUser)
	http.HandleFunc("/game_user_free", game.apiGameUserFree)

	log.Println("启动示例服务在 :5000")
	err := http.ListenAndServe(":5000", nil)
	return err
}
